==> 0.0.5.zip: sinqmotor/src/sinqController.cpp <==
/*
  This class contains the necessary changes to have an additional text fields
  for messages with each axis.

  Code lifted from Torsten Boegershausen ESS code.

  Mark Koennecke, March 2017

  Added code to manage an interMessageSleep

  Mark Koennecke, February 2024
*/

#include "sinqController.h"
#include "asynMotorController.h"
#include "epicsExport.h"
#include "iocsh.h"
#include "sinqAxis.h"
#include <errlog.h>

sinqController::sinqController(const char *portName, const char *SINQPortName,
                               int numAxes, const int &extraParams)
    : asynMotorController(
          portName, numAxes + 1, NUM_MOTOR_DRIVER_PARAMS + extraParams,
          0, // No additional interfaces beyond those in base class
          0, // No additional callback interfaces beyond those in base class
          ASYN_CANBLOCK | ASYN_MULTIDEVICE,
          1,    // autoconnect
          0, 0) // Default priority and stack size
{

    // Initialization of local variables
    asynStatus status = asynSuccess;

    // =========================================================================;

    // MOTOR_MESSAGE_TEXT corresponds to the PV definition inside
    // sinqn_asyn_motor.db. This text is used to forward status messages to
    // NICOS and in turn to the user.
    status =
        createParam("MOTOR_MESSAGE_TEXT", asynParamOctet, &motorMessageText_);
    if (status != asynSuccess) {
        asynPrint(this->pasynUserSelf, ASYN_TRACE_ERROR,
                  "%s => line %d:\nFATAL ERROR (creating a parameter failed "
                  "with %s)\n. Terminating IOC",
                  __PRETTY_FUNCTION__, __LINE__, stringifyAsynStatus(status));
        exit(-1);
    }

    // Internal parameter library entry which stores the movement target
    status = createParam("MOTOR_TARGET_POSITION", asynParamOctet,
                         &motorTargetPosition_);
    if (status != asynSuccess) {
        asynPrint(this->pasynUserSelf, ASYN_TRACE_ERROR,
                  "%s => line %d:\nFATAL ERROR (creating a parameter failed "
                  "with %s)\n. Terminating IOC",
                  __PRETTY_FUNCTION__, __LINE__, stringifyAsynStatus(status));
        exit(-1);
    }
}

sinqController::~sinqController(void) {
    /*
    Cleanup of the memory allocated in the asynMotorController constructor
    */
    free(this->pAxes_);
}

asynStatus sinqController::errMsgCouldNotParseResponse(const char *command,
                                                       const char *response,
                                                       int axisNo_,
                                                       const char *functionName,
                                                       int lineNumber) {
    asynPrint(lowLevelPortUser_, ASYN_TRACE_ERROR,
              "%s => line %d:\n Could not interpret response %s for "
              "command %s.\n",
              functionName, lineNumber, response, command);

    setStringParam(motorMessageText_,
                   "Could not interpret MCU response. Please "
                   "call the software support");
    setIntegerParam(motorStatusCommsError_, 1);
    return asynError;
}

asynStatus sinqController::paramLibAccessFailed(asynStatus status,
                                                const char *parameter,
                                                const char *functionName,
                                                int lineNumber) {

    if (status != asynSuccess) {
        // Log the error message and try to propagate it
        asynPrint(lowLevelPortUser_, ASYN_TRACE_ERROR,
                  "%s => line %d:\n Accessing the parameter library failed for "
                  "parameter %s",
                  functionName, lineNumber, parameter);
        setStringParam(
            motorMessageText_,
            "Accessing paramLib failed. Please call the software support.");
    }

    return status;
}

// Static pointers (valid for the entire lifetime of the IOC). The number behind
// the strings gives the integer number of each variant (see also method
// stringifyAsynStatus)
const char *asynSuccessStringified = "success";                             // 0
const char *asynTimeoutStringified = "timeout";                             // 1
const char *asynOverflowStringified = "overflow";                           // 2
const char *asynErrorStringified = "error";                                 // 3
const char *asynDisconnectedStringified = "disconnected";                   // 4
const char *asynDisabledStringified = "disabled";                           // 5
const char *asynParamAlreadyExistsStringified = "parameter already exists"; // 6
const char *asynParamNotFoundStringified = "parameter not found";           // 7
const char *asynParamWrongTypeStringified = "wrong type";                   // 8
const char *asynParamBadIndexStringified = "bad index";                     // 9
const char *asynParamUndefinedStringified = "parameter undefined"; // 10
const char *asynParamInvalidListStringified = "invalid list";      // 11

const char *sinqController::stringifyAsynStatus(asynStatus status) {
    // See
    // https://github.com/epics-modules/asyn/blob/master/asyn/asynDriver/asynDriver.h
    // and
    // https://github.com/epics-modules/asyn/blob/master/asyn/asynPortDriver/paramErrors.h
    // for the definition of the error codes
    switch (status) {
    case asynSuccess:
        return asynSuccessStringified;
    case asynTimeout:
        return asynTimeoutStringified;
    case asynOverflow:
        return asynOverflowStringified;
    case asynError:
        return asynErrorStringified;
    case asynDisconnected:
        return asynDisconnectedStringified;
    case asynDisabled:
        return asynDisabledStringified;
    case asynParamAlreadyExists:
        return asynParamAlreadyExistsStringified;
    case asynParamNotFound:
        return asynParamNotFoundStringified;
    case asynParamWrongType:
        return asynParamWrongTypeStringified;
    case asynParamBadIndex:
        return asynParamBadIndexStringified;
    case asynParamUndefined:
        return asynParamUndefinedStringified;
    case asynParamInvalidList:
        return asynParamInvalidListStringified;
    }

    errlogPrintf("%s => line %d:\nReached unreachable code.",
                 __PRETTY_FUNCTION__, __LINE__);
    return "unreachable code reached";
}

// =============================================================================
// Provide the setters to IOC shell
extern "C" {

asynStatus setOffsetMovTimeout(const char *portName, int axisNo,
                               double offsetMovTimeout) {

    sinqController *pC;
    pC = (sinqController *)findAsynPortDriver(portName);
    if (pC == nullptr) {
        errlogPrintf("%s => line %d:\nPort %s not found.", __PRETTY_FUNCTION__,
                     __LINE__, portName);
        return asynError;
    }

    asynMotorAxis *asynAxis = pC->getAxis(axisNo);
    sinqAxis *axis = dynamic_cast<sinqAxis *>(asynAxis);
    if (axis == nullptr) {
        errlogPrintf("%s => line %d:\nPAxis %d does not exist or is not an "
                     "instance of sinqAxis.",
                     __PRETTY_FUNCTION__, __LINE__, portName, axisNo);
    }

    return axis->setOffsetMovTimeout(offsetMovTimeout);
}

static const iocshArg setOffsetMovTimeoutArg0 = {"Controller port name",
                                                 iocshArgString};
static const iocshArg setOffsetMovTimeoutArg1 = {"Axis number", iocshArgInt};
static const iocshArg setOffsetMovTimeoutArg2 = {"Offset timeout for movement",
                                                 iocshArgDouble};
static const iocshArg *const setOffsetMovTimeoutArgs[] = {
    &setOffsetMovTimeoutArg0, &setOffsetMovTimeoutArg1,
    &setOffsetMovTimeoutArg2};
static const iocshFuncDef setOffsetMovTimeoutDef = {"setOffsetMovTimeout", 3,
                                                    setOffsetMovTimeoutArgs};

static void setOffsetMovTimeoutCallFunc(const iocshArgBuf *args) {
    setOffsetMovTimeout(args[0].sval, args[1].ival, args[1].dval);
}

// ============================================================================

asynStatus setScaleMovTimeout(const char *portName, int axisNo,
                              double scaleMovTimeout) {

    sinqController *pC;
    pC = (sinqController *)findAsynPortDriver(portName);
    if (pC == nullptr) {
        errlogPrintf("%s => line %d:\nPort %s not found.", __PRETTY_FUNCTION__,
                     __LINE__, portName);
        return asynError;
    }

    asynMotorAxis *asynAxis = pC->getAxis(axisNo);
    sinqAxis *axis = dynamic_cast<sinqAxis *>(asynAxis);
    if (axis == nullptr) {
        errlogPrintf("%s => line %d:\nPAxis %d does not exist or is not an "
                     "instance of sinqAxis.",
                     __PRETTY_FUNCTION__, __LINE__, portName, axisNo);
    }

    return axis->setScaleMovTimeout(scaleMovTimeout);
}

static const iocshArg setScaleMovTimeoutArg0 = {"Controller port name",
                                                iocshArgString};
static const iocshArg setScaleMovTimeoutArg1 = {"Axis number", iocshArgInt};
static const iocshArg setScaleMovTimeoutArg2 = {
    "Multiplier for calculated move time", iocshArgDouble};
static const iocshArg *const setScaleMovTimeoutArgs[] = {
    &setScaleMovTimeoutArg0, &setScaleMovTimeoutArg1, &setScaleMovTimeoutArg2};
static const iocshFuncDef setScaleMovTimeoutDef = {"setScaleMovTimeout", 3,
                                                   setScaleMovTimeoutArgs};

static void setScaleMovTimeoutCallFunc(const iocshArgBuf *args) {
    setScaleMovTimeout(args[0].sval, args[1].ival, args[1].dval);
}

static void sinqControllerRegister(void) {
    iocshRegister(&setOffsetMovTimeoutDef, setOffsetMovTimeoutCallFunc);
    iocshRegister(&setOffsetMovTimeoutDef, setOffsetMovTimeoutCallFunc);
}
epicsExportRegistrar(sinqControllerRegister);

} // extern C

==> 0.0.6.zip: sinqmotor/src/sinqController.cpp <==
/*
  This class contains the necessary changes to have an additional text fields
  for messages with each axis.

  Code lifted from Torsten Boegershausen ESS code.

  Mark Koennecke, March 2017

  Added code to manage an interMessageSleep

  Mark Koennecke, February 2024
*/

#include "sinqController.h"
#include "asynMotorController.h"
#include "epicsExport.h"
#include "iocsh.h"
#include "sinqAxis.h"
#include <errlog.h>

sinqController::sinqController(const char *portName, const char *SINQPortName,
                               int numAxes, const int &extraParams)
    : asynMotorController(
          portName, numAxes + 1, NUM_MOTOR_DRIVER_PARAMS + extraParams,
          0, // No additional interfaces beyond those in base class
          0, // No additional callback interfaces beyond those in base class
          ASYN_CANBLOCK | ASYN_MULTIDEVICE,
          1,    // autoconnect
          0, 0) // Default priority and stack size
{

    // Initialization of local variables
    asynStatus status = asynSuccess;

    // =========================================================================;

    // MOTOR_MESSAGE_TEXT corresponds to the PV definition inside
    // sinqn_asyn_motor.db. This text is used to forward status messages to
    // NICOS and in turn to the user.
    status =
        createParam("MOTOR_MESSAGE_TEXT", asynParamOctet, &motorMessageText_);
    if (status != asynSuccess) {
        asynPrint(this->pasynUserSelf, ASYN_TRACE_ERROR,
                  "%s => line %d:\nFATAL ERROR (creating a parameter failed "
                  "with %s)\n. Terminating IOC",
                  __PRETTY_FUNCTION__, __LINE__, stringifyAsynStatus(status));
        exit(-1);
    }

    // Internal parameter library entry which stores the movement target
    status = createParam("MOTOR_TARGET_POSITION", asynParamOctet,
                         &motorTargetPosition_);
    if (status != asynSuccess) {
        asynPrint(this->pasynUserSelf, ASYN_TRACE_ERROR,
                  "%s => line %d:\nFATAL ERROR (creating a parameter failed "
                  "with %s)\n. Terminating IOC",
                  __PRETTY_FUNCTION__, __LINE__, stringifyAsynStatus(status));
        exit(-1);
    }
}

sinqController::~sinqController(void) {
    /*
    Cleanup of the memory allocated in the asynMotorController constructor
    */
    free(this->pAxes_);
}

asynStatus sinqController::errMsgCouldNotParseResponse(const char *command,
                                                       const char *response,
                                                       int axisNo_,
                                                       const char *functionName,
                                                       int lineNumber) {
    asynPrint(lowLevelPortUser_, ASYN_TRACE_ERROR,
              "%s => line %d:\n Could not interpret response %s for "
              "command %s.\n",
              functionName, lineNumber, response, command);

    setStringParam(motorMessageText_,
                   "Could not interpret MCU response. Please "
                   "call the software support");
    setIntegerParam(motorStatusCommsError_, 1);
    return asynError;
}

asynStatus sinqController::paramLibAccessFailed(asynStatus status,
                                                const char *parameter,
                                                const char *functionName,
                                                int lineNumber) {

    if (status != asynSuccess) {
        // Log the error message and try to propagate it
        asynPrint(lowLevelPortUser_, ASYN_TRACE_ERROR,
                  "%s => line %d:\n Accessing the parameter library failed for "
                  "parameter %s",
                  functionName, lineNumber, parameter);
        setStringParam(
            motorMessageText_,
            "Accessing paramLib failed. Please call the software support.");
    }

    return status;
}

// Static pointers (valid for the entire lifetime of the IOC). The number behind
// the strings gives the integer number of each variant (see also method
// stringifyAsynStatus)
const char *asynSuccessStringified = "success";                             // 0
const char *asynTimeoutStringified = "timeout";                             // 1
const char *asynOverflowStringified = "overflow";                           // 2
const char *asynErrorStringified = "error";                                 // 3
const char *asynDisconnectedStringified = "disconnected";                   // 4
const char *asynDisabledStringified = "disabled";                           // 5
const char *asynParamAlreadyExistsStringified = "parameter already exists"; // 6
const char *asynParamNotFoundStringified = "parameter not found";           // 7
const char *asynParamWrongTypeStringified = "wrong type";                   // 8
const char *asynParamBadIndexStringified = "bad index";                     // 9
const char *asynParamUndefinedStringified = "parameter undefined"; // 10
const char *asynParamInvalidListStringified = "invalid list";      // 11

const char *sinqController::stringifyAsynStatus(asynStatus status) {
    // See
    // https://github.com/epics-modules/asyn/blob/master/asyn/asynDriver/asynDriver.h
    // and
    // https://github.com/epics-modules/asyn/blob/master/asyn/asynPortDriver/paramErrors.h
    // for the definition of the error codes
    // The pragma is necessary since the param lib error codes are "tacked on"
    // the enum, which results in compiler warnings otherwise.
#pragma GCC diagnostic ignored "-Wswitch"
    switch (status) {
    case asynSuccess:
        return asynSuccessStringified;
    case asynTimeout:
        return asynTimeoutStringified;
    case asynOverflow:
        return asynOverflowStringified;
    case asynError:
        return asynErrorStringified;
    case asynDisconnected:
        return asynDisconnectedStringified;
    case asynDisabled:
        return asynDisabledStringified;
    case asynParamAlreadyExists:
        return asynParamAlreadyExistsStringified;
    case asynParamNotFound:
        return asynParamNotFoundStringified;
    case asynParamWrongType:
        return asynParamWrongTypeStringified;
    case asynParamBadIndex:
        return asynParamBadIndexStringified;
    case asynParamUndefined:
        return asynParamUndefinedStringified;
    case asynParamInvalidList:
        return asynParamInvalidListStringified;
    }

    asynPrint(
        pasynUserSelf, ASYN_TRACE_ERROR,
        "%s => line %d:\nInput did not match any variant of asynStatus.\n",
        __PRETTY_FUNCTION__, __LINE__);

    return "Input did not match any variant of asynStatus";
}

// =============================================================================
// Provide the setters to IOC shell
extern "C" {

asynStatus setOffsetMovTimeout(const char *portName, int axisNo,
                               double offsetMovTimeout) {

    sinqController *pC;
    pC = (sinqController *)findAsynPortDriver(portName);
    if (pC == nullptr) {
        errlogPrintf("%s => line %d:\nPort %s not found.", __PRETTY_FUNCTION__,
                     __LINE__, portName);
        return asynError;
    }

    asynMotorAxis *asynAxis = pC->getAxis(axisNo);
    sinqAxis *axis = dynamic_cast<sinqAxis *>(asynAxis);
    if (axis == nullptr) {
        errlogPrintf("%s => line %d:\nAxis %d does not exist or is not an "
                     "instance of sinqAxis.",
                     __PRETTY_FUNCTION__, __LINE__, axisNo);
    }

    return axis->setOffsetMovTimeout(offsetMovTimeout);
}

static const iocshArg setOffsetMovTimeoutArg0 = {"Controller port name",
                                                 iocshArgString};
static const iocshArg setOffsetMovTimeoutArg1 = {"Axis number", iocshArgInt};
static const iocshArg setOffsetMovTimeoutArg2 = {"Offset timeout for movement",
                                                 iocshArgDouble};
static const iocshArg *const setOffsetMovTimeoutArgs[] = {
    &setOffsetMovTimeoutArg0, &setOffsetMovTimeoutArg1,
    &setOffsetMovTimeoutArg2};
static const iocshFuncDef setOffsetMovTimeoutDef = {"setOffsetMovTimeout", 3,
                                                    setOffsetMovTimeoutArgs};

static void setOffsetMovTimeoutCallFunc(const iocshArgBuf *args) {
    setOffsetMovTimeout(args[0].sval, args[1].ival, args[1].dval);
}

// ============================================================================

asynStatus setScaleMovTimeout(const char *portName, int axisNo,
                              double scaleMovTimeout) {

    sinqController *pC;
    pC = (sinqController *)findAsynPortDriver(portName);
    if (pC == nullptr) {
        errlogPrintf("%s => line %d:\nPort %s not found.", __PRETTY_FUNCTION__,
                     __LINE__, portName);
        return asynError;
    }

    asynMotorAxis *asynAxis = pC->getAxis(axisNo);
    sinqAxis *axis = dynamic_cast<sinqAxis *>(asynAxis);
    if (axis == nullptr) {
        errlogPrintf("%s => line %d:\nAxis %d does not exist or is not an "
                     "instance of sinqAxis.",
                     __PRETTY_FUNCTION__, __LINE__, axisNo);
    }

    return axis->setScaleMovTimeout(scaleMovTimeout);
}

static const iocshArg setScaleMovTimeoutArg0 = {"Controller port name",
                                                iocshArgString};
static const iocshArg setScaleMovTimeoutArg1 = {"Axis number", iocshArgInt};
static const iocshArg setScaleMovTimeoutArg2 = {
    "Multiplier for calculated move time", iocshArgDouble};
static const iocshArg *const setScaleMovTimeoutArgs[] = {
    &setScaleMovTimeoutArg0, &setScaleMovTimeoutArg1, &setScaleMovTimeoutArg2};
static const iocshFuncDef setScaleMovTimeoutDef = {"setScaleMovTimeout", 3,
                                                   setScaleMovTimeoutArgs};

static void setScaleMovTimeoutCallFunc(const iocshArgBuf *args) {
    setScaleMovTimeout(args[0].sval, args[1].ival, args[1].dval);
}

static void sinqControllerRegister(void) {
    iocshRegister(&setOffsetMovTimeoutDef, setOffsetMovTimeoutCallFunc);
    iocshRegister(&setScaleMovTimeoutDef, setScaleMovTimeoutCallFunc);
}
epicsExportRegistrar(sinqControllerRegister);

} // extern C
